use core::fmt;

pub trait WriteError: fmt::Debug {}

impl WriteError for () {}

pub enum SerializerError<WE>
	where WE: WriteError
{
	Write(WE),
	/// A unexpected error (should never get this).
	Unexpected(&'static str)
}

impl<WE> From<WE> for SerializerError<WE>
	where WE: WriteError
{
	fn from(err: WE) -> Self {
		SerializerError::Write(err)
	}
}

impl<WE> fmt::Debug for SerializerError<WE>
	where WE: WriteError
{
	fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
		use SerializerError::*;
		match self {
			Write(ref err) => write!(f, "write error: {:?}", err),
			Unexpected(ref err) => write!(f, "unexpected error: {}", err)
		}
	}
}