use crate::se::*;
use super::{GenericWriteBuffer, GenericReadBuffer};

pub struct GenericBufferWriter<T>
    where T: GenericWriteBuffer + GenericReadBuffer
{
    buf: T
}

impl<T> From<T> for GenericBufferWriter<T>
	where T: GenericWriteBuffer + GenericReadBuffer
{
	fn from(buf: T) -> Self {
		let mut gen_buf = Self { buf };
		// Ensure the buffer is empty.
		gen_buf.clear();
		gen_buf
	}
}

impl<T> GenericBufferWriter<T>
    where T: GenericWriteBuffer + GenericReadBuffer
{
    pub fn into_inner(self) -> T {
        self.buf
    }

    pub fn as_str(&self) -> &str {
		unsafe { core::str::from_utf8_unchecked(self.buf.as_slice()) }
	}

	#[cfg(any(feature = "std", feature = "alloc"))]	
	pub fn to_string(self) -> String {
		unsafe { String::from_utf8_unchecked(self.buf.to_vec()) }
	}

	#[inline]
	pub fn clear(&mut self) {
		self.buf.clear();
	}
}

impl<T> Write for &mut GenericBufferWriter<T>
where
	T: GenericWriteBuffer + GenericReadBuffer,
    T::Error: WriteError
{
	type Error = T::Error;

	#[inline]
	unsafe fn write_delim_unchecked(&mut self, delim: u8) -> Result<(), Self::Error> {
		self.buf.push_byte(delim)
	}

	#[inline]
	unsafe fn write_utf8_unchecked(&mut self, slice: &[u8]) -> Result<(), Self::Error> {
		self.buf.push_slice(slice)
	}
}